"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cli_flags_1 = require("cli-flags");
const cli_engine_config_1 = require("cli-engine-config");
const http_call_1 = require("http-call");
const help_1 = require("./help");
const pjson = require('../package.json');
const debug = require('debug')('cli-engine-command');
class Command {
    constructor(config) {
        this.hidden = false;
        this.aliases = [];
        this._version = pjson.version;
        this.parse = { flags: {} };
        this.config = cli_engine_config_1.buildConfig(config);
    }
    get id() {
        if (this.name)
            return this.name;
        let cmd = [];
        if (this.topic)
            cmd.push(this.topic);
        if (this.command)
            cmd.push(this.command);
        let id = cmd.join(':');
        if (id)
            return id;
        let ctor = this.constructor;
        return ctor.name;
    }
    static mock(...argv) {
        return __awaiter(this, void 0, void 0, function* () {
            argv.unshift('argv0', 'argv1');
            const cmd = (yield this.run({ argv, mock: true }));
            return {
                cmd,
                stdout: cmd.cli.stdout.output,
                stderr: cmd.cli.stderr.output,
            };
        });
    }
    static run(config) {
        return __awaiter(this, void 0, void 0, function* () {
            const cmd = new this(config);
            yield cmd._run();
            return cmd;
        });
    }
    _run() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                debug('initializing %s version: %s', this.id, this._version);
                debug('argv: %o', this.config.argv);
                yield this.init();
                debug('run');
                yield this.run();
                debug('done');
                yield this.cli.done();
            }
            catch (err) {
                this.cli.error(err);
            }
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.color = require('./color').color;
            const { CLI } = require('cli-ux');
            this.cli = new CLI({ debug: this.config.debug, mock: this.config.mock, errlog: this.config.errlog });
            this.http = http_call_1.HTTP.defaults({
                headers: {
                    'user-agent': `${this.config.name}/${this.config.version} (${this.config.platform}-${this.config
                        .arch}) node-${process.version}`,
                },
            });
            if (!this.config.mock)
                this.cli.handleUnhandleds();
            const { argv, flags, args } = yield cli_flags_1.parse(Object.assign({}, this.parse, { argv: this.config.argv.slice(2) }));
            this.flags = flags;
            this.argv = argv;
            this.args = args;
        });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    done() {
        return __awaiter(this, void 0, void 0, function* () {
            this.cli.done();
        });
    }
    buildHelp() {
        let help = new help_1.Help(this.config);
        return help.command(this);
    }
    buildHelpLine() {
        let help = new help_1.Help(this.config);
        return help.commandLine(this);
    }
}
exports.Command = Command;
